Stan Math Library: stan/math/prim/scal/prob/uniform_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

uniform_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_UNIFORM_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_UNIFORM_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_greater.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
14 #include <stan/math/prim/scal/meta/include_summand.hpp>
15 #include <boost/random/uniform_real_distribution.hpp>
16 #include <boost/random/variate_generator.hpp>
17 #include <cmath>
18
19 namespace stan {
20
21 namespace math {
22
23 // CONTINUOUS, UNIVARIATE DENSITIES
45 template <bool propto,
46 typename T_y, typename T_low, typename T_high>
47 typename return_type<T_y, T_low, T_high>::type
48 uniform_log(const T_y& y, const T_low& alpha, const T_high& beta) {
49 static const char* function("stan::math::uniform_log");
50 typedef typename stan::partials_return_type<T_y, T_low, T_high>::type
51 T_partials_return;
52
53 using stan::math::check_not_nan;
54 using stan::math::check_finite;
55 using stan::math::check_greater;
56 using stan::math::value_of;
57 using stan::math::check_consistent_sizes;
58 using std::log;
59
60 // check if any vectors are zero length
61 if (!(stan::length(y)
62 && stan::length(alpha)
63 && stan::length(beta)))
64 return 0.0;
65
66 // set up return value accumulator
67 T_partials_return logp(0.0);
68 check_not_nan(function, "Random variable", y);
69 check_finite(function, "Lower bound parameter", alpha);
70 check_finite(function, "Upper bound parameter", beta);
71 check_greater(function, "Upper bound parameter", beta, alpha);
72 check_consistent_sizes(function,
73 "Random variable", y,
74 "Lower bound parameter", alpha,
75 "Upper bound parameter", beta);
76
77 // check if no variables are involved and prop-to
78 if (!include_summand<propto, T_y, T_low, T_high>::value)
79 return 0.0;
80
81 VectorView<const T_y> y_vec(y);
82 VectorView<const T_low> alpha_vec(alpha);
83 VectorView<const T_high> beta_vec(beta);
84 size_t N = max_size(y, alpha, beta);
85
86 for (size_t n = 0; n < N; n++) {
87 const T_partials_return y_dbl = value_of(y_vec[n]);
88 if (y_dbl < value_of(alpha_vec[n])
89 || y_dbl > value_of(beta_vec[n]))
90 return LOG_ZERO;
91 }
92
93 VectorBuilder<include_summand<propto, T_low, T_high>::value,
94 T_partials_return, T_low, T_high>
95 inv_beta_minus_alpha(max_size(alpha, beta));
96 for (size_t i = 0; i < max_size(alpha, beta); i++)
97 if (include_summand<propto, T_low, T_high>::value)
98 inv_beta_minus_alpha[i]
99 = 1.0 / (value_of(beta_vec[i]) - value_of(alpha_vec[i]));
100
101 VectorBuilder<include_summand<propto, T_low, T_high>::value,
102 T_partials_return, T_low, T_high>
103 log_beta_minus_alpha(max_size(alpha, beta));
104 for (size_t i = 0; i < max_size(alpha, beta); i++)
105 if (include_summand<propto, T_low, T_high>::value)
106 log_beta_minus_alpha[i]
107 = log(value_of(beta_vec[i]) - value_of(alpha_vec[i]));
108
109 OperandsAndPartials<T_y, T_low, T_high>
110 operands_and_partials(y, alpha, beta);
111 for (size_t n = 0; n < N; n++) {
112 if (include_summand<propto, T_low, T_high>::value)
113 logp -= log_beta_minus_alpha[n];
114
115 if (!is_constant_struct<T_low>::value)
116 operands_and_partials.d_x2[n] += inv_beta_minus_alpha[n];
117 if (!is_constant_struct<T_high>::value)
118 operands_and_partials.d_x3[n] -= inv_beta_minus_alpha[n];
119 }
120 return operands_and_partials.value(logp);
121 }
122
123 template <typename T_y, typename T_low, typename T_high>
124 inline
125 typename return_type<T_y, T_low, T_high>::type
126 uniform_log(const T_y& y, const T_low& alpha, const T_high& beta) {
127 return uniform_log<false>(y, alpha, beta);
128 }
129 }
130 }
131 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
check_greater.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::uniform_log
return_type< T_y, T_low, T_high >::type uniform_log(const T_y &y, const T_low &alpha, const T_high &beta)
The log of a uniform density for the given y, lower, and upper bound.
Definition: uniform_log.hpp:48
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_greater
bool check_greater(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is strictly greater than low.
Definition: check_greater.hpp:84

[ Stan Home Page ] © 2011–2016, Stan Development Team.